// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 epicspeccombine

   Authors:  Aitor Ibarra - XMM-Newton SOC
         
   Description: Combines a list of spectrum files to create one combined spectrum file and the corresponding rmf, arf and bkg files
 
   Comments: 
 
  $Id: epicspeccombine.cc,v 1.3 2013/04/09 14:53:32 aibarra Exp $
*/

#include "epicspeccombine.h"
#include "EpicInstrInfo.h"
#include "EpicManip.h"
#include "RspManip.h"
#include "Dal.h"

#include <Tracer.h>
#include <errstr.h>
#include <map>
#include <algorithm>

using namespace std;

bool commensurate(std::vector<std::string>, std::vector<std::string>);


void epicspeccombine ()
{

	/*
	 * Read input parameters spectra, rmf and arf
	 */
	vector<string> pha = stringListParameter("pha");
	vector<string> rmf = stringListParameter("rmf");
	vector<string> arf = stringListParameter("arf");
	vector<string> bkg = stringListParameter("bkg");


	// Check that the number of input files are paired
	if ( ( pha.size() != rmf.size() ) ||
			( pha.size() != arf.size() ) )
		Sas::error("pairing","unpaired pha or rmf");

	if ((bkg.size() != 0 && pha.size() != bkg.size()) )
		Sas::error("pairing","unpaired pha or bkg");

	bool withback = true;
	if (bkg.size() == 0)
		withback = false;

	std::vector<string> epicSpectra = pha;
	epicSpectra.insert(epicSpectra.end(), bkg.begin(), bkg.end());

	/*
	 * Check  consistency between src spectra and ARFs
	 */
	commensurate(pha, rmf);

	/*
	 * Combine the EPIC camera source & background spectra
	 */

	 EpicInstrInfo *e = new EpicInstrInfo();
	 int tlmin = 0 ;
	 int tlmax = 0 ;

	for (unsigned int i = 0 ; i < epicSpectra.size(); ++i)
	{

		// Get BACKSCALE and EXPOSURE keywords from MOS spectra
		DataSet *set = dataSetServer->open(epicSpectra[i], DataSet::Read);

		double backscale = set->table("SPECTRUM")->attribute("BACKSCAL")->asReal();
		double exposure = set->table("SPECTRUM")->attribute("EXPOSURE")->asReal();
		string obsID = set->attribute("OBS_ID")->asString();
		string expID = set->attribute("EXPIDSTR")->asString();
		string instr = set->attribute("INSTRUME")->asString();
		tlmin = set->table("SPECTRUM")->column("CHANNEL")->attribute("TLMIN")->asInt();
		tlmax = set->table("SPECTRUM")->column("CHANNEL")->attribute("TLMAX")->asInt();
		int npi = set->table("SPECTRUM")->rows();
		//EpicInstrInfo *e = new EpicInstrInfo(obsID, Xmm::toInstrument(instr),
		//		 expID);
		EpicInstrInfo *ee = new EpicInstrInfo();
		 ee->setObsID(obsID);
		 ee->setInstr(Xmm::toInstrument(instr));
		 ee->setExposureID(expID);
		 ee->setFileName(epicSpectra[i]);
		 ee->setExposure(exposure);
		 ee->setBackscale(backscale);
		 ee->setTLMin(tlmin);
		 ee->setTLMax(tlmax);
		 ee->setNPI(npi);
		 if (std::find(pha.begin(), pha.end(), epicSpectra[i]) != pha.end())
			 ee->setType("SRC");
		 else
			 ee->setType("BKG");
		 e->pushSpectra(ee);

	}

	//Sum source spectra
	EpicManip epicSpectraManip;

	vector<EpicInstrInfo*> srcS = e->getSpectra("SRC");

	errstr << "Adding source spectra "<< Sas::message(Msg::App,Msg::Verbose);
	epicSpectraManip.addSpectra(srcS);


	//epicSpectraManip.addSpectra();
	//Sum background spectra
	vector<EpicInstrInfo*> bkgS = e->getSpectra("BKG");

	errstr << "Adding background spectra "<< Sas::message(Msg::App,Msg::Verbose);
	epicSpectraManip.addSpectra(bkgS);

	// Employ the average of the exposure time of the individual
	// spectra as exposure time of the combined MOS spectrum


	/*
	 * Combine the EPIC camera source & background spectra
	 */


	/*
	 * Merge the EPIC effective areas and redistribution matrices
	 */
	errstr << "combining rmf and arf " << Sas::message(Msg::App,Msg::Verbose);
	RspManip rspmanip;
	rspmanip.addResponses(arf,rmf,e);

	/*
	 * Merge the EPIC effective areas and redistribution matrices
	 */


}

bool commensurate(std::vector<std::string> sfiles,
                      std::vector<std::string> rfiles)
{
        // Check the spectra have consistent binning
	DataSet *set = dataSetServer->open(sfiles[0], DataSet::Read);

	int rows = set->table("SPECTRUM")->rows();
	dataSetServer->close(set);
	for (unsigned int i = 1 ; i < sfiles.size(); ++i)
	{
		DataSet *set = dataSetServer->open(sfiles[i], DataSet::Read);
		int nrows= set->table("SPECTRUM")->rows();
		if (rows!=nrows)
			errstr << "Different PI channel interval"
			<< Sas::error("specChannelIntervalError");

                dataSetServer->close(set);

	}

        // Check the RMFs have consistent binning
        set = dataSetServer->open(rfiles[0], DataSet::Read);

        rows = set->table("MATRIX")->rows();
        dataSetServer->close(set);
        for (unsigned int i = 1 ; i < rfiles.size(); ++i)
        {
                DataSet *set = dataSetServer->open(rfiles[i], DataSet::Read);
                int nrows= set->table("MATRIX")->rows();
                if (rows!=nrows)
                  errstr << "Different energy bins in response files"
                         << Sas::error("rmfEnergiesError");

                dataSetServer->close(set);

        }

	return true;
}
